"""@package docstring
B-ASIC Signal Module.
"""
from typing import TYPE_CHECKING, Optional

from b_asic.graph_component import TypeName
from b_asic.abstract_graph_component import AbstractGraphComponent

if TYPE_CHECKING:
	from b_asic import OutputPort, InputPort

class Signal(AbstractGraphComponent):
	"""A connection between two ports."""
	_source: "OutputPort"
	_destination: "InputPort"

	def __init__(self, src: Optional["OutputPort"] = None, dest: Optional["InputPort"] = None, **kwds):
		super().__init__(**kwds)
		self._source = src
		self._destination = dest

	@property
	def source(self) -> "OutputPort":
		"""Returns the source OutputPort of the signal."""
		return self._source

	@property
	def destination(self) -> "InputPort":
		"""Returns the destination InputPort of the signal."""
		return self._destination

	@source.setter
	def source(self, src: "OutputPort") -> None:
		"""Sets the value of the source OutputPort of the signal."""
		self._source = src

	@destination.setter
	def destination(self, dest: "InputPort") -> None:
		"""Sets the value of the destination InputPort of the signal."""
		self._destination = dest

	@property
	def type_name(self) -> TypeName:
		return "s"

	def disconnect_source(self) -> None:
		"""Disconnects the source OutputPort of the signal."""
		self._source = None

	def disconnect_destination(self) -> None:
		"""Disconnects the destination InputPort of the signal."""
		self._destination = None
